(*****************************************************************************)
(*                                                                           *)
(*  MRUList.PAS                                                              *)
(*  08/08/94                                                                 *)
(*  Brad Stowers                                                             *)
(*       CIS:      72733,3374                                                *)
(*       Internet: brad.stowers@delta.com                                    *)
(*                                                                           *)
(*  This code is released to the public domain.  Use at your own risk!       *)
(*  No support or guarantees are made or implied.  If you have problems      *)
(*  feel free to contact me at one of the above addresses and I will help    *)
(*  all I can.  If you make improvements, PLEASE share them on BPASCAL.      *)
(*                                                                           *)
(*****************************************************************************)
(*                                                                           *)
(*  Most Recently Used File List                                             *)
(*    Adapted from an article by Steven Palmer in the June 1993 issue        *)
(*    of DOS/Windows Developer's Journal.  All credit goes to Mr. Palmer.    *)
(*                                                                           *)
(*  Adds filenames to your File menu for a list of recently used files.      *)
(*  Also, two simple procedures to save and load the files to an INI file.   *)
(*                                                                           *)
(*  This file contains both the library code for the DLL and the import      *)
(*  unit.  Compile this file twice, first with the MAKEDLL defined and again *)
(*  without it.  This will create both the MRULIST.DLL and MRULIST.TPW files.*)
(*                                                                           *)
(*****************************************************************************)

{ $DEFINE MAKEDLL}

{$IFDEF MAKEDLL}
library MRUList;

uses WinProcs, WinTypes, Strings;

{$I MRULIST.INC}

type
  tFileItem = array [0..127] of char;
{ Defaults }
const
  MaxItems:  integer = 4;
  MDIWnd:    hWnd    = 0;
  NumFiles:  integer = 0;
  NumOnMenu: integer = 0;
{ Variables }
var
  Files:    array[1..9] of tFileItem;


{ Utilities }
function minIntOf(X,Y:Integer):Integer;inline(
  $58/                   {pop ax      ;y}
  $5A/                   {pop dx      ;x}
  $39/$D0/               {cmp ax,dx}
  $7E/+02/               {jle *+2}
  $89/$D0);              {mov ax,dx}


{ Exported Procedures and Functions }
procedure InitMRUList(Items: integer; MDIChild: hWnd); export;
begin
  if (Items > 0) and (Items < 10) then MaxItems := Items
  else if (Items < 1) then MaxItems := 1 else MaxItems := 9;
  MDIWnd := MDIChild;
end;


procedure UpdateMRUList(Wnd: hWnd); export;
var
  Menu:     hMenu;
  MenuNum,
  x, Count: integer;
  NewItem:  array [0..129] of char;
  WVS_Rec:  record
    Num:      integer;
    Item:     pChar;
  end;
begin
  MenuNum := 0;
  if MDIWnd <> 0 then
    if HiWord(SendMessage(MDIWnd, WM_MDIGETACTIVE, 0, 0)) > 0 then inc(MenuNum);
  Menu := GetSubMenu(GetMenu(Wnd), MenuNum);
  Count := GetMenuItemCount(Menu)-1;
  for x := 1 to MinIntOf(NumFiles,NumOnMenu) do begin
    WVS_Rec.Num := x;
    WVS_Rec.Item := Files[x];
    WVSPrintF(NewItem, '&%d %s', WVS_Rec);
    ModifyMenu(Menu, MRU_FILE1+x-1, MF_STRING, MRU_FILE1+x-1, NewItem);
  end;
  if NumOnMenu < NumFiles then begin
    if NumOnMenu = 0 then
      InsertMenu(Menu, Count, MF_SEPARATOR or MF_BYPOSITION, 0, NIL)
    else
      dec(Count);
    for x := NumOnMenu+1 to NumFiles do begin
      WVS_Rec.Num := x;
      WVS_Rec.Item := Files[x];
      WVSPrintF(NewItem, '&%d %s', WVS_Rec);
      InsertMenu(Menu, Count, MF_STRING or MF_BYPOSITION, MRU_FILE1+x-1, NewItem);
    end;
  end else
    for x := NumFiles to NumOnMenu do
      DeleteMenu(Menu, MRU_FILE1+x, MF_BYCOMMAND);
  if (NumOnMenu > 0) and (NumFiles = 0) then
    DeleteMenu(Menu, 999, MF_BYCOMMAND);
  NumOnMenu := NumFiles;
  DrawMenubar(Wnd);
end;


procedure AddMRUItem(Wnd: hWnd; Name: pChar); export;
var
  x, y: integer;
begin
  if Name=NIL then exit;
  y := NumFiles;
  for x := 1 to NumFiles do begin
    if (StrIComp(Files[x], Name) = 0) then begin
      y := x-1;
      break;
    end;
  end;
  if (y = NumFiles) and (NumFiles < MaxItems) then
    inc(NumFiles);
  if y = MaxItems then begin
    dec(y);
  end;

  for x := y downto 1 do
    StrCopy(Files[x+1], Files[x]);
  StrCopy(Files[1], Name);
  UpdateMRUList(Wnd);
end;


procedure DeleteMRUItem(Wnd: hWnd; Name: pChar); export;
var
  x, y: integer;
begin
  if Name = NIL then exit;
  for x := 1 to NumFiles do
    if (StrIComp(Files[x], Name) = 0) then begin
      for y := x+1 to NumFiles do
        StrCopy(Files[y-1], Files[y]);
      dec(NumFiles);
      UpdateMRUList(Wnd);
    end;
end;


function GetMRUItem(ID: integer; Str: pChar): boolean; export;
var
  RetVal: boolean;
  Index: integer;
begin
  Index := (ID-MRU_FILE1+1);
  RetVal := Index <= NumFiles;
  if RetVal then
    StrCopy(Str, Files[Index])
  else
    Str[0] := #0;
  GetMRUItem := RetVal;
end;


procedure AppendMRUList(Wnd: hWnd); export;
var
  Menu:    hMenu;
  MenuNum,
  x, Count   : integer;
  NewItem    : array [0..129] of char;
  WVS_Rec: record
    Num: integer;
    Item: pChar;
  end;
begin
  MenuNum := 0;
  if MDIWnd <> 0 then
    if HiWord(SendMessage(MDIWnd, WM_MDIGETACTIVE, 0, 0)) > 0 then inc(MenuNum);
  Menu := GetMenu(Wnd);
  Menu := GetSubMenu(Menu, MenuNum);
  Count := GetMenuItemCount(Menu)-1;
  for x := 1 to NumFiles do begin
    WVS_Rec.Num := x;
    WVS_Rec.Item := Files[x];
    WVSPrintF(NewItem, '&%d %s', WVS_Rec);
    InsertMenu(Menu, Count, MF_STRING or MF_BYPOSITION, MRU_FILE1+x-1, NewItem);
    Inc(Count);
  end;
  InsertMenu(Menu, Count, MF_SEPARATOR or MF_BYPOSITION, 0, NIL);
  NumOnMenu := NumFiles;
end;


procedure LoadMRUList(Wnd: hWnd; AINIFile, AKeyName: pChar); export;
var
  x,y: integer;
  INIFile: array[0..127] of char;
  KeyName: array[0..127] of char;
  Temp: array[0..6] of char;
  NewItem    : array [0..129] of char;
begin
  if (not assigned(AINIFile)) or (assigned(AINIFile) and (AINIFile[0] = #0)) then
    StrLCopy(INIFile, 'WIN.INI', SizeOf(INIFile)-1)
  else
    StrLCopy(INIFile, AINIFile, SizeOf(INIFile)-1);
  if (not assigned(AKeyName)) or (assigned(AINIFile) and (AKeyName[0] = #0)) then
    StrLCopy(KeyName, 'MRU Files', SizeOf(KeyName)-1)
  else
    StrLCopy(KeyName, AKeyName, SizeOf(KeyName)-1);
  for x := MRU_FILE1+MaxItems-1 downto MRU_FILE1 do begin
    y := x-MRU_FILE1+1;
    WVSPrintF(Temp, 'File_%d', y);
    if GetPrivateProfileString(KeyName, Temp, '', NewItem, SizeOf(NewItem)-1, INIFile) > 0 then
      AddMRUItem(Wnd, NewItem);
  end;
end;


procedure SaveMRUList(AINIFile, AKeyName: pChar); export;
var
  x,y: integer;
  INIFile: array[0..127] of char;
  KeyName: array[0..127] of char;
  Temp: array[0..6] of char;
  NewItem    : array [0..129] of char;
  WVS_Rec: record
    Num: integer;
    Item: pChar;
  end;
begin
  if (not assigned(AINIFile)) then
    StrLCopy(INIFile, 'WIN.INI', SizeOf(INIFile)-1)
  else
    StrLCopy(INIFile, AINIFile, SizeOf(INIFile)-1);
  if (not assigned(AKeyName)) then
    StrLCopy(KeyName, 'MRU Files', SizeOf(KeyName)-1)
  else
    StrLCopy(KeyName, AKeyName, SizeOf(KeyName)-1);
  for x := MRU_FILE1 to MRU_FILE1+MaxItems-1 do begin
    y := x-MRU_FILE1+1;
    WVSPrintF(Temp, 'File_%d', y);
    if GetMRUItem(x, NewItem) then
      WritePrivateProfileString(KeyName, Temp, NewItem, INIFile)
    else
      WritePrivateProfileString(KeyName, Temp, '', INIFile)
  end;
end;


exports
  InitMRUList index 1,
  AddMRUItem index 2,
  DeleteMRUItem index 3,
  GetMRUItem index 4,
  AppendMRUList index 5,
  UpdateMRUList index 6,
  LoadMRUList index 7,
  SaveMRUList index 8;

{$ELSE}


{ Unit wrapper for DLL version. }

unit MRUList;

interface

uses WinTypes;

procedure InitMRUList(Items: integer; MDIChild: hWnd);
procedure UpdateMRUList(Wnd: hWnd);
procedure AddMRUItem(Wnd: hWnd; Name: pChar);
procedure DeleteMRUItem(Wnd: hWnd; Name: pChar);
function  GetMRUItem(ID: integer; Str: pChar): boolean;
procedure AppendMRUList(Wnd: hWnd);
procedure LoadMRUList(Wnd: hWnd; AINIFile, AKeyName: pChar);
procedure SaveMRUList(AINIFile, AKeyName: pChar);

implementation

procedure InitMRUList; external 'MRULIST' index 1;
procedure AddMRUItem; external 'MRULIST' index 2;
procedure DeleteMRUItem; external 'MRULIST' index 3;
function  GetMRUItem; external 'MRULIST' index 4;
procedure AppendMRUList; external 'MRULIST' index 5;
procedure UpdateMRUList; external 'MRULIST' index 6;
procedure LoadMRUList; external 'MRULIST' index 7;
procedure SaveMRUList; external 'MRULIST' index 8;

{$ENDIF}

begin
end.
