(*****************************************************************************)
(*                                                                           *)
(*  ooMRU.PAS                                                                *)
(*  08/08/94                                                                 *)
(*  Brad Stowers                                                             *)
(*       CIS:      72733,3374                                                *)
(*       Internet: brad.stowers@delta.com                                    *)
(*                                                                           *)
(*  This code is released to the public domain.  Use at your own risk!       *)
(*  No support or guarantees are made or implied.  If you have problems      *)
(*  feel free to contact me at one of the above addresses and I will help    *)
(*  all I can.  If you make improvements, PLEASE share them on BPASCAL.      *)
(*  This code was inspired by an article on the same topic in the            *)
(*  Windows/DOS Developers Journal, Novemeber 1993, by Mark Szamrej.  It is  *)
(*  not as "all-encompassing" as what is provided in that article, simply    *)
(*  because I didn't need all that.                                          *)
(*                                                                           *)
(*****************************************************************************)
(*                                                                           *)
(*  Most Recently Used File List                                             *)
(*    Adapted from an article by Steven Palmer in the June 1993 issue        *)
(*    of DOS/Windows Developer's Journal.  All credit goes to Mr. Palmer.    *)
(*                                                                           *)
(*  This file provides an object for the MRU File List to make it even       *)
(*  simpler to add it to your own applications.                              *)
(*                                                                           *)
(*****************************************************************************)
(*                                                                           *)
(*  To add an MRU List to the MDI File app provided with BP 7 (the file is   *)
(* \BP\EXAMPLES\WIN\OWL\MFILEAPP.PAS), do the following:                     *)
(*                                                                           *)
(* Search for:  TMDIFileWindow = object(TMDIWindow)                          *)
(* and add below it:                                                         *)
(*     MRUList: pMRUList;                                                    *)
(*     destructor Done; virtual;                                             *)
(*     procedure wmCommand(var Msg: TMessage);                               *)
(*                                                                           *)
(* Search for:  destructor TFileEditor.Done;                                 *)
(* and after "begin" add:                                                    *)
(*     PMDIFileWindow(Parent)^.MRUList^.Add(FileName);                       *)
(*                                                                           *)
(* Search for:  function TFileEditor.GetClassName: PChar;                    *)
(* and after the function add:                                               *)
(*     destructor TMDIFileWindow.Done;                                       *)
(*     begin                                                                 *)
(*       CloseChildren;                                                      *)
(*       MRUList^.Save('MRULIST.INI', 'MRU Files');                          *)
(*       dispose(MRUList, Done);                                             *)
(*       inherited Done;                                                     *)
(*      end;                                                                 *)
(*      procedure TMDIFileWindow.wmCommand(var Msg: TMessage);               *)
(*      var FileName: array[0..127] of char;                                 *)
(*      begin                                                                *)
(*        if (Msg.lParamHi = 0) and ((Msg.wParam >= MRU_FILE1) and           *)
(*                                   (Msg.wParam <= MRU_FILE9)) then begin   *)
(*         if MRUList^.GetFile(Msg.wParam, FileName) then                    *)
(*          Application^.MakeWindow(New(PFileEditor, Init(@Self, FileName))) *)
(*        end else                                                           *)
(*          inherited wmCommand(Msg);                                        *)
(*      end;                                                                 *)
(*                                                                           *)
(* Search for:  TMDIWindow.SetupWindow;                                      *)
(* and after it add:                                                         *)
(*      MRUList := New(pMRUList, Load(5, hWindow, ClientWnd^.hWindow,        *)
(*                     'MRULIST.INI', 'MRU Files'));                         *)
(*                                                                           *)
(*****************************************************************************)

unit ooMRU;

interface

uses Objects, WinTypes;

{$I MRULIST.INC}

type
  pMRUList = ^tMRUList;
  tMRUList = object(tObject)
    { If your app is not MDI, just pass a zero for the MDIChild parameter. }
    { If it is MDI, pass TMDIWindow^.ClientWnd^.hWindow.                   }
    constructor Init(MaxItems: integer; MenuWnd, MDIChild: hWnd);
    constructor Load(MaxItems: integer; MenuWnd, MDIChild: hWnd; AINIFile, AKeyName: pChar);
    procedure   Save(AINIFile, AKeyName: pChar);
    procedure   Add(AFile: pChar);
    procedure   Delete(AFile: pChar);
    function    GetFile(AnID: integer; AFile: pChar): boolean;
    { This is provided for apps that need to change their main menu during run-time }
    procedure   ChangeMenu(NewMenu: hMenu);
    { Used mainly internally, but provided "just in case." }
    procedure   Refresh;
  private
    Parent: hWnd;
  end;

implementation

uses MRUList;

constructor tMRUList.Init(MaxItems: integer; MenuWnd, MDIChild: hWnd);
begin
  inherited Init;
  InitMRUList(MaxItems, MDIChild);
  Parent := MenuWnd;
end;


procedure tMRUList.Add(AFile: pChar);
begin
  AddMRUItem(Parent, AFile);
end;


procedure tMRUList.Delete(AFile: pChar);
begin
  DeleteMRUItem(Parent, AFile);
end;


function tMRUList.GetFile(AnID: integer; AFile: pChar): boolean;
begin
  GetFile := GetMRUItem(AnID, AFile);
end;


procedure tMRUList.Save(AINIFile, AKeyName: pChar);
begin
  SaveMRUList(AINIFile, AKeyName);
end;


constructor tMRUList.Load(MaxItems: integer; MenuWnd, MDIChild: hWnd; AINIFile, AKeyName: pChar);
begin
  inherited Init;
  Parent := MenuWnd;
  InitMRUList(MaxItems, MDIChild);
  LoadMRUList(Parent, AINIFile, AKeyName);
end;


procedure tMRUList.ChangeMenu(NewMenu: hMenu);
begin
  Parent := NewMenu;
  AppendMRUList(Parent);
end;


procedure tMRUList.Refresh;
begin
  UpdateMRUList(Parent);
end;


end.
